@@ -248,6 +248,7 @@ chomp(char *str)
 *  cmd -- command to send
 *  buf -- buffer for reply
 *  len -- length of buffer
+*  qid -- Sendmail queue identifier
 * %RETURNS:
 *  0 if all went well, -1 on error.
 * %DESCRIPTION:
@@ -257,16 +258,21 @@ int
 MXCommand(char const *sockname,
 	  char const *cmd,
 	  char *buf,
-	  int len)
+	  int len,
+	  char const *qid)
 {
     int fd;
     struct sockaddr_un addr;
     int nread;
     int n;
 
+    if (!qid || !*qid) {
+	qid = "NOQUEUE";
+    }
+
     fd = socket(AF_LOCAL, SOCK_STREAM, 0);
     if (fd < 0) {
-	syslog(LOG_ERR, "MXCommand: socket: %m");
+	syslog(LOG_ERR, "%s: MXCommand: socket: %m", qid);
 	return MD_TEMPFAIL;
     }
 
@@ -275,14 +281,14 @@ MXCommand(char const *sockname,
     strncpy(addr.sun_path, sockname, sizeof(addr.sun_path) - 1);
 
     if (connect(fd, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-	syslog(LOG_ERR, "MXCommand: connect: %m: Is multiplexor running?");
+	syslog(LOG_ERR, "%s: MXCommand: connect: %m: Is multiplexor running?", qid);
 	close(fd);
 	return MD_TEMPFAIL;
     }
 
     n = writestr(fd, cmd);
     if (n < 0) {
-	syslog(LOG_ERR, "MXCommand: write: %m: Is multiplexor running?");
+	syslog(LOG_ERR, "%s: MXCommand: write: %m: Is multiplexor running?", qid);
 	close(fd);
 	return MD_TEMPFAIL;
     }
@@ -290,7 +296,7 @@ MXCommand(char const *sockname,
     /* Now read the answer */
     nread = readn(fd, buf, len-1);
     if (nread < 0) {
-	syslog(LOG_ERR, "MXCommand: read: %m: Is multiplexor running?");
+	syslog(LOG_ERR, "%s: MXCommand: read: %m: Is multiplexor running?", qid);
 	close(fd);
 	return MD_TEMPFAIL;
     }
@@ -299,7 +305,7 @@ MXCommand(char const *sockname,
     if (nread == len-1) {
 	char slop[SMALLBUF];
 	if (readn(fd, slop, SMALLBUF) > 0) {
-	    syslog(LOG_WARNING, "MXCommand: Overlong reply from multiplexor was truncated!");
+	    syslog(LOG_WARNING, "%s: MXCommand: Overlong reply from multiplexor was truncated!", qid);
 	    /* Read all the way to EOF */
 	    while (readn(fd, slop, SMALLBUF) > 0);
 	}
@@ -319,12 +325,12 @@ MXCommand(char const *sockname,
 *  Queries multiplexor for number of free slaves.
 ***********************************************************************/
 int
-MXCheckFreeSlaves(char const *sockname)
+MXCheckFreeSlaves(char const *sockname, char const *qid)
 {
     char ans[SMALLBUF];
     int slaves;
 
-    if (MXCommand(sockname, "free\n", ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
+    if (MXCommand(sockname, "free\n", ans, SMALLBUF-1, qid) < 0) return MD_TEMPFAIL;
 
     if (sscanf(ans, "%d", &slaves) != 1) return MD_TEMPFAIL;
     return slaves;
@@ -358,13 +364,13 @@ MXScanDir(char const *sockname,
 	return MD_TEMPFAIL;
     }
 
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1, qid) < 0) return MD_TEMPFAIL;
 
     if (!strcmp(ans, "ok\n")) return 0;
 
     len = strlen(ans);
     if (len > 0 && ans[len-1] == '\n') ans[len-1] = 0;
-    syslog(LOG_ERR, "Error from multiplexor: %s", ans);
+    syslog(LOG_ERR, "%s: Error from multiplexor: %s", qid, ans);
     return MD_TEMPFAIL;
 }
 
@@ -509,15 +515,20 @@ percent_encode_command(int term_with_newline, char *out, int outlen, ...)
 * %ARGUMENTS:
 *  ans -- answer from multiplexor
 *  msg -- buffer for holding error message, at least SMALLBUF chars
+*  qid -- Sendmail queue ID
 * %RETURNS:
 *  1 if it's OK to accept connections from this host; 0 if not, -1 if error.
 *  If connection is rejected, error message *may* be set.
 ***********************************************************************/
 static int
-munch_mx_return(char *ans, char *msg)
+munch_mx_return(char *ans, char *msg, char const *qid)
 {
     size_t len;
 
+    if (!qid || !*qid) {
+	qid = "NOQUEUE";
+    }
+
     if (!strcmp(ans, "ok -1\n")) return MD_TEMPFAIL;
     if (!strcmp(ans, "ok 1\n")) return MD_CONTINUE;
     if (!strcmp(ans, "ok 2\n")) return MD_ACCEPT_AND_NO_MORE_FILTERING;
@@ -554,7 +565,7 @@ munch_mx_return(char *ans, char *msg)
     }
 
     if (len > 0 && ans[len-1] == '\n') ans[len-1] = 0;
-    syslog(LOG_ERR, "Error from multiplexor: %s", ans);
+    syslog(LOG_ERR, "%s: Error from multiplexor: %s", qid, ans);
     return MD_TEMPFAIL;
 }
 
@@ -605,8 +616,8 @@ MXRelayOK(char const *sockname,
     if (percent_encode_command(1, cmd, sizeof(cmd), "relayok", ip, name, port_string, myip, daemon_port_string, NULL) < 0) {
 	return MD_TEMPFAIL;
     }
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
-    return munch_mx_return(ans, msg);
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1, NULL) < 0) return MD_TEMPFAIL;
+    return munch_mx_return(ans, msg, NULL);
 }
 
 /**********************************************************************
@@ -662,8 +673,8 @@ MXHeloOK(char const *sockname,
     if (percent_encode_command(1, cmd, sizeof(cmd), "helook", ip, name, helo, port_string, myip, daemon_port_string, NULL) < 0) {
 	return MD_TEMPFAIL;
     }
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
-    return munch_mx_return(ans, msg);
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1, NULL) < 0) return MD_TEMPFAIL;
+    return munch_mx_return(ans, msg, NULL);
 }
 
 
@@ -741,8 +752,8 @@ MXSenderOK(char const *sockname,
     /* Add newline */
     strcat(cmd, "\n");
 
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
-    return munch_mx_return(ans, msg);
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1, qid) < 0) return MD_TEMPFAIL;
+    return munch_mx_return(ans, msg, qid);
 }
 
 /**********************************************************************
@@ -836,8 +847,8 @@ MXRecipientOK(char const *sockname,
     /* Add newline */
     strcat(cmd, "\n");
 
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
-    return munch_mx_return(ans, msg);
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1, qid) < 0) return MD_TEMPFAIL;
+    return munch_mx_return(ans, msg, qid);
 }
 
 /**********************************************************************
@@ -1214,107 +1225,6 @@ make_listening_socket(char const *str, int backlog, int must_be_unix)
     return sock;
 }
 
-/**********************************************************************
-* %FUNCTION: connect_to_socket
-* %ARGUMENTS:
-*  str -- a string of the form:
-*                   /path/to/sock   (assumed to be unix:/path/to/sock)
-*                   unix:/path/to/sock
-*                   local:/path/to/sock
-*                   inet:port  (host defaults to LOOPBACK)
-*                   inet6:port (host defaults to in6addr_loopback)
-* %RETURNS:
-*  Connects on specified UNIX-domain or TCP socket.
-* %DESCRIPTION:
-*  Utility function for opening a connected socket socket.
-***********************************************************************/
-int
-connect_to_socket(char const *str)
-{
-    char const *path;
-    int sock;
-    struct sockaddr_in addr_in;
-    struct sockaddr_un addr_un;
-    int port;
-
-    if (!strncmp(str, "inet:", 5)) {
-	path = str+5;
-	if (sscanf(path, "%d", &port) != 1 ||
-	    port < 1 ||
-	    port > 65535) {
-	    syslog(LOG_ERR, "connect_to_socket: Invalid port %s", path);
-	    errno = EINVAL;
-	    return -1;
-	}
-
-	sock = socket(PF_INET, SOCK_STREAM, 0);
-	if (sock < 0) {
-	    syslog(LOG_ERR, "connect_to_socket: socket: %m");
-	    return -1;
-	}
-
-	addr_in.sin_family = AF_INET;
-	addr_in.sin_port = htons(port);
-	addr_in.sin_addr.s_addr = (uint32_t) htonl(INADDR_LOOPBACK);
-	if (connect(sock, (struct sockaddr *) &addr_in, sizeof(addr_in)) < 0) {
-	    syslog(LOG_ERR, "connect_to_socket: connect: %m");
-	    close(sock);
-	    return -1;
-	}
-    } else if (!strncmp(str, "inet6:", 6)) {
-#if defined(AF_INET6)
-	struct sockaddr_in6 addr_in6;
-	path = str+5;
-	if (sscanf(path, "%d", &port) != 1 ||
-	    port < 1 ||
-	    port > 65535) {
-	    syslog(LOG_ERR, "connect_to_socket: Invalid port %s", path);
-	    errno = EINVAL;
-	    return -1;
-	}
-
-	sock = socket(PF_INET6, SOCK_STREAM, 0);
-	if (sock < 0) {
-	    syslog(LOG_ERR, "connect_to_socket: socket: %m");
-	    return -1;
-	}
-
-	addr_in6.sin6_family = AF_INET6;
-	addr_in6.sin6_port = htons(port);
-	addr_in6.sin6_addr = in6addr_loopback;
-	if (connect(sock, (struct sockaddr *) &addr_in6, sizeof(addr_in6)) < 0) {
-	    syslog(LOG_ERR, "connect_to_socket: connect: %m");
-	    close(sock);
-	    return -1;
-	}
-#else
-	syslog(LOG_ERR, "Cannot specify inet6 socket: No IPv6 support");
-	fprintf(stderr, "Cannot specify inet6 socket: No IPv6 support\n");
-	return -1;
-#endif
-    } else {
-	/* Assume unix-domain socket */
-	path = str;
-	if (!strncmp(str, "unix:", 5)) path = str+5;
-	else if (!strncmp(str, "local:", 6)) path = str+6;
-	sock = socket(AF_LOCAL, SOCK_STREAM, 0);
-	if (sock < 0) {
-	    syslog(LOG_ERR, "connect_to_socket: socket: %m");
-	    return -1;
-	}
-
-	memset(&addr_un, 0, sizeof(addr_un));
-	addr_un.sun_family = AF_LOCAL;
-	strncpy(addr_un.sun_path, path, sizeof(addr_un.sun_path)-1);
-
-	if (connect(sock, (struct sockaddr *) &addr_un, SUN_LEN(&addr_un)) < 0) {
-	    syslog(LOG_ERR, "connect_to_socket: connect: %m");
-	    close(sock);
-	    return -1;
-	}
-    }
-    return sock;
-}
 /**********************************************************************
 * %FUNCTION: do_delay
 * %ARGUMENTS:
